@@ -1,6 +1,7 @@
-import torch
 import numpy as np
 import pandas as pd
+import torch
+from torch import nn
 from torch import optim
 from pathlib import Path
 from typing import Union, List, Tuple, Collection
@@ -21,24 +22,28 @@ class ModelWrapper:
 	Examples
 	--------
 	>>> from nlpx.model.wrapper import ModelWrapper
-	>>> model_wrapper = ModelWrapper()
-	>>> model_wrapper.train(model, train_set, val_set, collate_fn)
+	>>> model_wrapper = ModelWrapper(model)
+	>>> model_wrapper.train(train_set, val_set, collate_fn)
 	>>> model_wrapper.logits(X_test)
 	"""
 	
-	def __init__(self, model_path: Union[str, Path] = None,
-	             device=torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')):
-		self.device = device
-		self.model = torch.load(model_path, map_location=device) if model_path else None
+	def __init__(self, model_or_path: Union[nn.Module, str, Path], device: torch.device = None):
+		if isinstance(model_or_path, nn.Module):
+			self.model, self.best_model = model_or_path, model_or_path
+		elif isinstance(model_or_path, (str, Path)):
+			self.model = torch.load(model_or_path, map_location=device)
+			self.best_model = self.model
+		self.device = device or torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')
 	
-	def train(self, model, train_set: Dataset, collate_fn=None, max_iter=100,
+	def train(self, train_set: Dataset, collate_fn=None, max_iter=100,
 	          optimizer: Union[type, optim.Optimizer] = None, scheduler: LRScheduler = None,
 	          learning_rate=0.001, T_max: int = 0,
 	          batch_size=64, num_workers=0,
 	          pin_memory: bool = False, pin_memory_device: str = "",
 	          persistent_workers: bool = False,
 	          early_stopping_rounds: int = 10,
-	          print_per_rounds: int = 1, show_progress=False):
+	          print_per_rounds: int = 1, checkpoint_per_rounds: int = 0, checkpoint_name='model.pt',
+	          show_progress=False, eps=1e-5):
 		trainer = Trainer(max_iter, optimizer, scheduler, learning_rate, T_max,
 		                  batch_size,
 		                  num_workers,
@@ -46,20 +51,37 @@ class ModelWrapper:
 		                  persistent_workers,
 		                  early_stopping_rounds,  # 早停，等10轮决策，评价指标不在变化，停止
 		                  print_per_rounds,
+		                  checkpoint_per_rounds,
+		                  checkpoint_name,
 		                  self.device)
-		self.model = trainer.train(model, train_set, collate_fn, show_progress)
+		self.best_model = trainer.train(self.model, train_set, collate_fn, show_progress, eps)
 	
 	def logits(self, X: torch.Tensor):
-		self.model.eval()
+		self.best_model.eval()
 		with torch.no_grad():
-			logits = self.model(X)
+			logits = self.best_model(X)
 		return logits
 	
-	def save(self, model_path: Union[str, Path] = './best_model.pt'):
-		torch.save(self.model, model_path)
+	def save(self, best_model_path: Union[str, Path] = './best_model.pt', 
+	         last_model_path: Union[str, Path] = './last_model.pt', save_mode: str = 'both'):
+		"""
+		:param best_model_path: 
+		:param last_model_path: 
+		:param save_mode: "both" or "best" or "last"
+		:return: 
+		"""
+		assert save_mode in ('both', 'best', 'last')
+		if save_mode == 'both':
+			torch.save(self.model, last_model_path)
+			torch.save(self.best_model, best_model_path)
+		elif save_mode == 'best':
+			torch.save(self.best_model, best_model_path)
+		elif save_mode == 'last':
+			torch.save(self.model, last_model_path)
 	
 	def load(self, model_path: Union[str, Path] = './best_model.pt'):
 		self.model = torch.load(model_path, map_location=self.device)
+		self.best_model = self.model
 
 
 class SimpleModelWrapper(ModelWrapper):
@@ -67,16 +89,16 @@ class SimpleModelWrapper(ModelWrapper):
 	Examples
 	--------
 	>>> from nlpx.model.wrapper import SimpleModelWrapper
-	>>> model_wrapper = SimpleModelWrapper()
-	>>> model_wrapper.train(model, X, y, collate_fn)
+	>>> model_wrapper = SimpleModelWrapper(model)
+	>>> model_wrapper.train(X, y, collate_fn)
 	>>> model_wrapper.logits(X_test)
 	"""
 	
-	def __init__(self, model_path: Union[str, Path] = None,
-	             device=torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')):
-		super().__init__(model_path, device)
+	def __init__(self, model_or_path: Union[nn.Module, str, Path],
+	             device: torch.device = None):
+		super().__init__(model_or_path, device)
 	
-	def train(self, model, X: Union[torch.Tensor, np.ndarray, List], y: Union[torch.LongTensor, np.ndarray, List],
+	def train(self, X: Union[torch.Tensor, np.ndarray, List], y: Union[torch.LongTensor, np.ndarray, List],
 	          collate_fn=None, max_iter=100,
 	          optimizer: Union[type, optim.Optimizer] = None, scheduler: LRScheduler = None,
 	          learning_rate=0.001, T_max: int = 0,
@@ -84,7 +106,8 @@ class SimpleModelWrapper(ModelWrapper):
 	          pin_memory: bool = False, pin_memory_device: str = "",
 	          persistent_workers: bool = False,
 	          early_stopping_rounds: int = 10,
-	          print_per_rounds: int = 1, show_progress=False):
+	          print_per_rounds: int = 1, checkpoint_per_rounds: int = 0, checkpoint_name='model.pt',
+	          show_progress=False, eps=1e-5):
 		trainer = SimpleTrainer(max_iter, optimizer, scheduler, learning_rate, T_max,
 		                        batch_size,
 		                        num_workers,
@@ -92,8 +115,10 @@ class SimpleModelWrapper(ModelWrapper):
 		                        persistent_workers,
 		                        early_stopping_rounds,  # 早停，等10轮决策，评价指标不在变化，停止
 		                        print_per_rounds,
+		                        checkpoint_per_rounds,
+		                        checkpoint_name,
 		                        self.device)
-		self.model = trainer.train(model, X, y, collate_fn, show_progress)
+		self.best_model = trainer.train(self.model, X, y, collate_fn, show_progress, eps)
 
 
 class ClassModelWrapper(ModelWrapper):
@@ -101,19 +126,18 @@ class ClassModelWrapper(ModelWrapper):
 	Examples
 	--------
 	>>> from nlpx.model.wrapper import ClassModelWrapper
-	>>> model_wrapper = ClassModelWrapper(classes=classes)
-	>>> model_wrapper.train(model, train_set, val_set, collate_fn)
+	>>> model_wrapper = ClassModelWrapper(model, classes=classes)
+	>>> model_wrapper.train(train_set, val_set, collate_fn)
 	>>> model_wrapper.predict(X_test)
 	>>> model_wrapper.evaluate(test_set)
 	"""
 	
-	def __init__(self, model_path: Union[str, Path] = None, classes: Collection[str] = None,
-	             device=torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')):
+	def __init__(self, model_or_path: Union[nn.Module, str, Path], classes: Collection[str] = None,
+	             device: torch.device = None):
+		super().__init__(model_or_path, device)
 		self.classes = classes
-		self.device = device
-		self.model = torch.load(model_path, map_location=device) if model_path else None
 	
-	def train(self, model, train_set: Dataset, val_set: Dataset = None, collate_fn=None, max_iter=100,
+	def train(self, train_set: Dataset, val_set: Dataset = None, collate_fn=None, max_iter=100,
 	          optimizer: Union[type, optim.Optimizer] = None, scheduler: LRScheduler = None,
 	          learning_rate=0.001, T_max: int = 0,
 	          batch_size=64, eval_batch_size=128,
@@ -121,7 +145,8 @@ class ClassModelWrapper(ModelWrapper):
 	          pin_memory: bool = False, pin_memory_device: str = "",
 	          persistent_workers: bool = False,
 	          early_stopping_rounds: int = 10,
-	          print_per_rounds: int = 1, show_progress=False):
+	          print_per_rounds: int = 1, checkpoint_per_rounds: int = 0, checkpoint_name='model.pt',
+	          show_progress=False, eps=1e-5):
 		if val_set:
 			trainer = EvalClassTrainer(max_iter, optimizer, scheduler, learning_rate, T_max,
 			                           batch_size, eval_batch_size,
@@ -130,8 +155,10 @@ class ClassModelWrapper(ModelWrapper):
 			                           persistent_workers,
 			                           early_stopping_rounds,  # 早停，等10轮决策，评价指标不在变化，停止
 			                           print_per_rounds,
+			                           checkpoint_per_rounds,
+			                           checkpoint_name,
 			                           self.device)
-			self.model = trainer.train(model, train_set, val_set, collate_fn, show_progress)
+			self.best_model = trainer.train(self.model, train_set, val_set, collate_fn, show_progress, eps)
 		else:
 			trainer = ClassTrainer(max_iter, optimizer, scheduler, learning_rate, T_max,
 			                       batch_size, num_workers,
@@ -139,8 +166,10 @@ class ClassModelWrapper(ModelWrapper):
 			                       persistent_workers,
 			                       early_stopping_rounds,  # 早停，等10轮决策，评价指标不在变化，停止
 			                       print_per_rounds,
+			                       checkpoint_per_rounds,
+			                       checkpoint_name,
 			                       self.device)
-			self.model = trainer.train(model, train_set, collate_fn, show_progress)
+			self.best_model = trainer.train(self.model, train_set, collate_fn, show_progress, eps)
 	
 	def predict(self, X: torch.Tensor):
 		logits = self.logits(X)
@@ -163,7 +192,7 @@ class ClassModelWrapper(ModelWrapper):
 	
 	def evaluate(self, val_set: Dataset, batch_size=64, num_workers=0, collate_fn=None):
 		val_loader = DataLoader(dataset=val_set, batch_size=batch_size, num_workers=num_workers, collate_fn=collate_fn)
-		_, acc = evaluate(self.model, val_loader, self.device)
+		_, acc = evaluate(self.best_model, val_loader, self.device)
 		return acc
 
 
@@ -172,17 +201,17 @@ class SimpleClassModelWrapper(ClassModelWrapper):
 	Examples
 	--------
 	>>> from nlpx.model.wrapper import SimpleClassModelWrapper
-	>>> model_wrapper = SimpleClassModelWrapper(classes=classes)
-	>>> model_wrapper.train(model, X, y val_data, collate_fn)
+	>>> model_wrapper = SimpleClassModelWrapper(model, classes=classes)
+	>>> model_wrapper.train(X, y val_data, collate_fn)
 	>>> model_wrapper.predict(X_test)
 	>>> model_wrapper.evaluate(X_test, y_test)
 	"""
 	
-	def __init__(self, model_path: Union[str, Path] = None, classes: Collection[str] = None,
-	             device=torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')):
-		super().__init__(model_path, classes, device)
+	def __init__(self, model_or_path: Union[nn.Module, str, Path], classes: Collection[str] = None,
+	             device: torch.device = None):
+		super().__init__(model_or_path, classes, device)
 	
-	def train(self, model, X: Union[torch.Tensor, np.ndarray, List], y: Union[torch.LongTensor, np.ndarray, List],
+	def train(self, X: Union[torch.Tensor, np.ndarray, List], y: Union[torch.LongTensor, np.ndarray, List],
 	          val_data: Tuple[Union[torch.Tensor, np.ndarray, List], Union[torch.LongTensor, np.ndarray, List]] = None,
 	          collate_fn=None, max_iter=100,
 	          optimizer: Union[type, optim.Optimizer] = None, scheduler: LRScheduler = None,
@@ -192,7 +221,8 @@ class SimpleClassModelWrapper(ClassModelWrapper):
 	          pin_memory: bool = False, pin_memory_device: str = "",
 	          persistent_workers: bool = False,
 	          early_stopping_rounds: int = 10,
-	          print_per_rounds: int = 1, show_progress=False):
+	          print_per_rounds: int = 1, checkpoint_per_rounds: int = 0, checkpoint_name='model.pt',
+	          show_progress=False, eps=1e-5):
 		if val_data:
 			trainer = EvalSimpleClassTrainer(max_iter, optimizer, scheduler, learning_rate, T_max,
 			                                 batch_size, eval_batch_size,
@@ -201,8 +231,10 @@ class SimpleClassModelWrapper(ClassModelWrapper):
 			                                 persistent_workers,
 			                                 early_stopping_rounds,
 			                                 print_per_rounds,
+			                                 checkpoint_per_rounds,
+			                                 checkpoint_name,
 			                                 self.device)
-			self.model = trainer.train(model, X, y, val_data, collate_fn, show_progress)
+			self.best_model = trainer.train(self.model, X, y, val_data, collate_fn, show_progress, eps)
 		else:
 			trainer = SimpleClassTrainer(max_iter, optimizer, scheduler, learning_rate, T_max,
 			                             batch_size, num_workers,
@@ -210,8 +242,10 @@ class SimpleClassModelWrapper(ClassModelWrapper):
 			                             persistent_workers,
 			                             early_stopping_rounds,
 			                             print_per_rounds,
+			                             checkpoint_per_rounds,
+			                             checkpoint_name,
 			                             self.device)
-			self.model = trainer.train(model, X, y, collate_fn, show_progress)
+			self.best_model = trainer.train(self.model, X, y, collate_fn, show_progress, eps)
 	
 	def evaluate(self, X: Union[torch.Tensor, np.ndarray, List], y: Union[torch.LongTensor, np.ndarray, List],
 	             batch_size=64, num_workers=0, collate_fn=None):
@@ -229,20 +263,20 @@ class TextModelWrapper(SimpleClassModelWrapper):
 	Examples
 	--------
 	>>> from nlpx.model.wrapper import TextModelWrapper
-	>>> model_wrapper = TextModelWrapper(tokenize_vec, classes=classes)
-	>>> model_wrapper.train(model, train_texts, y_train val_data, collate_fn)
+	>>> model_wrapper = TextModelWrapper(model, tokenize_vec, classes=classes)
+	>>> model_wrapper.train(train_texts, y_train val_data, collate_fn)
 	>>> model_wrapper.predict(test_texts)
 	>>> model_wrapper.evaluate(test_texts, y_test)
 	"""
 	
-	def __init__(self, tokenize_vec: Union[
-		BaseTokenizer, PaddingTokenizer, SimpleTokenizer, Tokenizer, TokenizeVec, TokenEmbedding],
-	             model_path: Union[str, Path] = None, classes: Collection[str] = None,
-	             device=torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')):
-		super().__init__(model_path, classes, device)
+	def __init__(self, model_or_path: Union[nn.Module, str, Path], 
+	             tokenize_vec: Union[BaseTokenizer, TokenizeVec, TokenEmbedding],
+	             classes: Collection[str] = None,
+	             device: torch.device = None):
+		super().__init__(model_or_path, classes, device)
 		self.tokenize_vec = tokenize_vec
 	
-	def train(self, model, texts: Union[Collection[str], np.ndarray, pd.Series],
+	def train(self, texts: Union[Collection[str], np.ndarray, pd.Series],
 	          y: Union[torch.LongTensor, np.ndarray, List],
 	          val_data: Tuple[
 		          Union[Collection[str], np.ndarray, pd.Series], Union[torch.LongTensor, np.ndarray, List]] = None,
@@ -254,13 +288,15 @@ class TextModelWrapper(SimpleClassModelWrapper):
 	          pin_memory: bool = False, pin_memory_device: str = "",
 	          persistent_workers: bool = False,
 	          early_stopping_rounds: int = 10,
-	          print_per_rounds: int = 1, n_jobs=-1, show_progress=False):
+	          print_per_rounds: int = 1, checkpoint_per_rounds: int = 0, checkpoint_name='model.pt',
+	          n_jobs=-1, show_progress=False, eps=1e-5):
 		X = self.get_vec(texts, max_length=max_length, n_jobs=n_jobs)
 		if val_data:
 			val_data = (self.get_vec(val_data[0], max_length=max_length, n_jobs=-1), val_data[1])
-		super().train(model, X, y, val_data, collate_fn, max_iter, optimizer, scheduler, learning_rate, T_max,
+		super().train(X, y, val_data, collate_fn, max_iter, optimizer, scheduler, learning_rate, T_max,
 		              batch_size, eval_batch_size, num_workers, num_eval_workers, pin_memory, pin_memory_device,
-		              persistent_workers, early_stopping_rounds, print_per_rounds, show_progress)
+		              persistent_workers, early_stopping_rounds, print_per_rounds, checkpoint_per_rounds,
+		              checkpoint_name, show_progress, eps)
 	
 	def predict(self, texts: Collection[str], max_length: int = None, n_jobs=-1):
 		logits = self.logits(texts, max_length, n_jobs=n_jobs)
@@ -317,19 +353,19 @@ class SplitTextModelWrapper(TextModelWrapper):
 	Examples
 	--------
 	>>> from nlpx.model.wrapper import SplitTextModelWrapper
-	>>> model_wrapper = SplitTextModelWrapper(tokenize_vec, classes=classes)
-	>>> model_wrapper.train(model, texts, y, collate_fn)
+	>>> model_wrapper = SplitTextModelWrapper(model, tokenize_vec, classes=classes)
+	>>> model_wrapper.train(texts, y, collate_fn)
 	>>> model_wrapper.predict(test_texts)
 	>>> model_wrapper.evaluate(test_texts, y_test)
 	"""
 	
-	def __init__(self, tokenize_vec: Union[
-		BaseTokenizer, PaddingTokenizer, SimpleTokenizer, Tokenizer, TokenizeVec, TokenEmbedding],
-	             model_path: Union[str, Path] = None, classes: Collection[str] = None,
-	             device=torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')):
-		super().__init__(tokenize_vec, model_path, classes, device)
+	def __init__(self, model_or_path: Union[nn.Module, str, Path],
+	             tokenize_vec: Union[BaseTokenizer, TokenizeVec, TokenEmbedding],
+	             classes: Collection[str] = None,
+	             device: torch.device = None):
+		super().__init__(model_or_path, tokenize_vec, classes, device)
 	
-	def train(self, model, texts: Union[Collection[str], np.ndarray, pd.Series],
+	def train(self, texts: Union[Collection[str], np.ndarray, pd.Series],
 	          y: Union[torch.LongTensor, np.ndarray, List],
 	          max_length: int = None, eval=True, val_size=0.2, random_state=None, collate_fn=None, max_iter=100,
 	          optimizer: Union[type, optim.Optimizer] = None, scheduler: LRScheduler = None,
@@ -339,7 +375,8 @@ class SplitTextModelWrapper(TextModelWrapper):
 	          pin_memory: bool = False, pin_memory_device: str = "",
 	          persistent_workers: bool = False,
 	          early_stopping_rounds: int = 10,
-	          print_per_rounds: int = 1, n_jobs=-1, show_progress=False):
+	          print_per_rounds: int = 1, checkpoint_per_rounds: int = 0, checkpoint_name='model.pt',
+	          n_jobs=-1, show_progress=False, eps=1e-5):
 		X = self.get_vec(texts, max_length=max_length, n_jobs=n_jobs)
 		if eval:
 			trainer = SplitClassTrainer(max_iter, optimizer, scheduler, learning_rate, T_max,
@@ -349,8 +386,10 @@ class SplitTextModelWrapper(TextModelWrapper):
 			                            persistent_workers,
 			                            early_stopping_rounds,
 			                            print_per_rounds,
+			                            checkpoint_per_rounds,
+			                            checkpoint_name,
 			                            self.device)
-			self.model = trainer.train(model, X, y, val_size, random_state, collate_fn, show_progress)
+			self.best_model = trainer.train(self.model, X, y, val_size, random_state, collate_fn, show_progress, eps)
 		else:
 			trainer = SimpleClassTrainer(max_iter, optimizer, scheduler, learning_rate, T_max,
 			                             batch_size, num_workers,
@@ -358,8 +397,10 @@ class SplitTextModelWrapper(TextModelWrapper):
 			                             persistent_workers,
 			                             early_stopping_rounds,
 			                             print_per_rounds,
+			                             checkpoint_per_rounds,
+			                             checkpoint_name,
 			                             self.device)
-			self.model = trainer.train(model, X, y, collate_fn, show_progress)
+			self.best_model = trainer.train(self.model, X, y, collate_fn, show_progress, eps)
 
 
 class PaddingTextModelWrapper(ClassModelWrapper):
@@ -367,20 +408,19 @@ class PaddingTextModelWrapper(ClassModelWrapper):
 	Examples
 	--------
 	>>> from nlpx.model.wrapper import PaddingTextModelWrapper
-	>>> model_wrapper = PaddingTextModelWrapper(tokenizer, classes=classes)
-	>>> model_wrapper.train(model, train_texts, y_train val_data)
+	>>> model_wrapper = PaddingTextModelWrapper(model, tokenizer, classes=classes)
+	>>> model_wrapper.train(train_texts, y_train val_data)
 	>>> model_wrapper.predict(test_texts)
 	>>> model_wrapper.evaluate(test_texts, y_test)
 	"""
-	
-	def __init__(self, tokenizer: Union[PaddingTokenizer, SimpleTokenizer, Tokenizer],
-	             model_path: Union[str, Path] = None,
+	def __init__(self, model_or_path: Union[nn.Module, str, Path],
+	             tokenizer: Union[PaddingTokenizer, SimpleTokenizer, Tokenizer],
 	             classes: Collection[str] = None,
-	             device=torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')):
-		super().__init__(model_path, classes, device)
+	             device: torch.device = None):
+		super().__init__(model_or_path, classes, device)
 		self.tokenizer = tokenizer
 	
-	def train(self, model, texts: Union[Collection[str], np.ndarray, pd.Series],
+	def train(self, texts: Union[Collection[str], np.ndarray, pd.Series],
 	          y: Union[torch.LongTensor, np.ndarray, List],
 	          val_data: Tuple[Union[Collection[str], np.ndarray, pd.Series], Union[torch.LongTensor, np.ndarray, List]] = None,
 	          max_length: int = None, max_iter=100,
@@ -391,7 +431,8 @@ class PaddingTextModelWrapper(ClassModelWrapper):
 	          pin_memory: bool = False, pin_memory_device: str = "",
 	          persistent_workers: bool = False,
 	          early_stopping_rounds: int = 10,
-	          print_per_rounds: int = 1, show_progress=False):
+	          print_per_rounds: int = 1, checkpoint_per_rounds: int = 0, checkpoint_name='model.pt',
+	          show_progress=False, eps=1e-5):
 		X = self.tokenizer.batch_encode(texts, padding=False)
 		train_set = TokenDataset(X, y)
 		val_set = None
@@ -399,12 +440,13 @@ class PaddingTextModelWrapper(ClassModelWrapper):
 			X_val = self.tokenizer.batch_encode(val_data[0], padding=False)
 			val_set = TokenDataset(X_val, val_data[1])
 		
-		super().train(model, train_set, val_set, collate_fn=PaddingTokenCollator(self.tokenizer.pad, max_length),
+		super().train(self.model, train_set, val_set, collate_fn=PaddingTokenCollator(self.tokenizer.pad, max_length),
 		              max_iter=max_iter, optimizer=optimizer, scheduler=scheduler, learning_rate=learning_rate,
 		              T_max=T_max, batch_size=batch_size, num_workers=num_workers, pin_memory=pin_memory,
 		              pin_memory_device=pin_memory_device, persistent_workers=persistent_workers,
 		              early_stopping_rounds=early_stopping_rounds, print_per_rounds=print_per_rounds,
-		              show_progress=show_progress)
+		              checkpoint_per_rounds=checkpoint_per_rounds, checkpoint_name=checkpoint_name,
+		              show_progress=show_progress, eps=eps)
 	
 	def predict(self, texts: Collection[str], max_length: int = None):
 		logits = self.logits(texts, max_length)
@@ -452,13 +494,13 @@ class SplitPaddingTextModelWrapper(PaddingTextModelWrapper):
 	>>> model_wrapper.evaluate(test_texts, y_test)
 	"""
 	
-	def __init__(self, tokenizer: Union[PaddingTokenizer, SimpleTokenizer, Tokenizer],
-	             model_path: Union[str, Path] = None,
+	def __init__(self, model_or_path: Union[nn.Module, str, Path],
+	             tokenizer: Union[PaddingTokenizer, SimpleTokenizer, Tokenizer],
 	             classes: Collection[str] = None,
-	             device=torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')):
-		super().__init__(tokenizer, model_path, classes, device)
+	             device: torch.device = None):
+		super().__init__(model_or_path, tokenizer, classes, device)
 	
-	def train(self, model, texts: Union[Collection[str], np.ndarray, pd.Series],
+	def train(self, texts: Union[Collection[str], np.ndarray, pd.Series],
 	          y: Union[torch.LongTensor, np.ndarray, List],
 	          max_length: int = None, eval=True, val_size=0.2, random_state=None, max_iter=100,
 	          optimizer: Union[type, optim.Optimizer] = None, scheduler: LRScheduler = None,
@@ -468,7 +510,8 @@ class SplitPaddingTextModelWrapper(PaddingTextModelWrapper):
 	          pin_memory: bool = False, pin_memory_device: str = "",
 	          persistent_workers: bool = False,
 	          early_stopping_rounds: int = 10,
-	          print_per_rounds: int = 1, show_progress=False):
+	          print_per_rounds: int = 1, checkpoint_per_rounds: int = 0, checkpoint_name='model.pt',
+	          show_progress=False, eps=1e-5):
 		val_data = None
 		if eval:
 			X_train, X_test, y_train, y_test = train_test_split(texts, y, test_size=val_size, random_state=random_state)
@@ -476,6 +519,7 @@ class SplitPaddingTextModelWrapper(PaddingTextModelWrapper):
 		else:
 			X_train, y_train = texts, y
 		
-		super().train(model, X_train, y_train, val_data, max_length, max_iter, optimizer, scheduler, learning_rate,
+		super().train(X_train, y_train, val_data, max_length, max_iter, optimizer, scheduler, learning_rate,
 		              T_max, batch_size, eval_batch_size, num_workers, num_eval_workers, pin_memory, pin_memory_device,
-		              persistent_workers, early_stopping_rounds, print_per_rounds, show_progress)
+		              persistent_workers, early_stopping_rounds, print_per_rounds, checkpoint_per_rounds,
+		              checkpoint_name, show_progress, eps)
