@@ -9,7 +9,10 @@ from torch.optim.lr_scheduler import LRScheduler, CosineAnnealingLR
 from sklearn.model_selection import train_test_split
 from nlpx import log_utils
 
-E = 1e-5
+
+def cal_count(y):
+	shape = y.shape
+	return shape[0] if len(shape) == 1 else shape[0] * shape[1]
 
 
 def cal_correct(logits: torch.Tensor, y: torch.Tensor):
@@ -24,19 +27,29 @@ def evaluate(model, val_loader, device):
 	total_loss, correct = [torch.Tensor([0.0]) for _ in range(2)]
 	model.eval()
 	with torch.no_grad():
-		for batch in val_loader:
-			batch = [x.to(device) if torch.is_tensor(x) else x for x in batch]
-			y = batch[-1]
-			logits, loss = model(*batch)
-			total_loss += loss
-			total += y.size(0) if len(y.size()) == 1 else y.size(0) * y.size(1)
-			correct += cal_correct(logits, y)
-			steps += 1
+		if hasattr(model, 'fit'):
+			for batch in val_loader:
+				batch = [x.to(device) if torch.is_tensor(x) else x for x in batch]
+				y = batch[-1]
+				logits, loss = model.fit(*batch)
+				total_loss += loss
+				total += cal_count(y)
+				correct += cal_correct(logits, y)
+				steps += 1
+		else:
+			for batch in val_loader:
+				batch = [x.to(device) if torch.is_tensor(x) else x for x in batch]
+				y = batch[-1]
+				logits, loss = model(*batch)
+				total_loss += loss
+				total += cal_count(y)
+				correct += cal_correct(logits, y)
+				steps += 1
 	return (total_loss.item() / steps), (correct.item() / total)
 
 
 def do_train(model, batch, optimizer, device):
-	batch = [x.to(device) if torch.is_tensor(x) else x for x in batch] if isinstance(batch, Tuple) else batch.to(device)
+	batch = [x.to(device) if torch.is_tensor(x) else x for x in batch]
 	logits, loss = model(*batch)
 	optimizer.zero_grad()
 	loss.backward()
@@ -44,8 +57,17 @@ def do_train(model, batch, optimizer, device):
 	return loss
 
 
+def do_fit(model, batch, optimizer, device):
+	batch = [x.to(device) if torch.is_tensor(x) else x for x in batch]
+	logits, loss = model.fit(*batch)
+	optimizer.zero_grad()
+	loss.backward()
+	optimizer.step()
+	return loss
+
+
 def do_train_scheduler(model, batch, optimizer, device, scheduler: LRScheduler):
-	batch = [x.to(device) if torch.is_tensor(x) else x for x in batch] if isinstance(batch, Tuple) else batch.to(device)
+	batch = [x.to(device) if torch.is_tensor(x) else x for x in batch]
 	logits, loss = model(*batch)
 	optimizer.zero_grad()
 	loss.backward()
@@ -54,6 +76,16 @@ def do_train_scheduler(model, batch, optimizer, device, scheduler: LRScheduler):
 	return loss
 
 
+def do_fit_scheduler(model, batch, optimizer, device, scheduler: LRScheduler):
+	batch = [x.to(device) if torch.is_tensor(x) else x for x in batch]
+	logits, loss = model.fit(*batch)
+	optimizer.zero_grad()
+	loss.backward()
+	optimizer.step()
+	scheduler.step()
+	return loss
+
+
 def do_train_acc(model, batch, optimizer, device):
 	batch = [x.to(device) if torch.is_tensor(x) else x for x in batch]
 	y = batch[-1]
@@ -61,7 +93,17 @@ def do_train_acc(model, batch, optimizer, device):
 	optimizer.zero_grad()
 	loss.backward()
 	optimizer.step()
-	return loss, y.size(0) if len(y.size()) == 1 else y.size(0) * y.size(1), cal_correct(logits, y)
+	return loss, cal_count(y), cal_correct(logits, y)
+
+
+def do_fit_acc(model, batch, optimizer, device):
+	batch = [x.to(device) if torch.is_tensor(x) else x for x in batch]
+	y = batch[-1]
+	logits, loss = model.fit(*batch)
+	optimizer.zero_grad()
+	loss.backward()
+	optimizer.step()
+	return loss, cal_count(y), cal_correct(logits, y)
 
 
 def do_train_scheduler_acc(model, batch, optimizer, device, scheduler: LRScheduler):
@@ -72,16 +114,33 @@ def do_train_scheduler_acc(model, batch, optimizer, device, scheduler: LRSchedul
 	loss.backward()
 	optimizer.step()
 	scheduler.step()
-	return loss, y.size(0) if len(y.size()) == 1 else y.size(0) * y.size(1), cal_correct(logits, y)
+	return loss, cal_count(y), cal_correct(logits, y)
+
+
+def do_fit_scheduler_acc(model, batch, optimizer, device, scheduler: LRScheduler):
+	batch = [x.to(device) if torch.is_tensor(x) else x for x in batch]
+	y = batch[-1]
+	logits, loss = model.fit(*batch)
+	optimizer.zero_grad()
+	loss.backward()
+	optimizer.step()
+	scheduler.step()
+	return loss, cal_count(y), cal_correct(logits, y)
 
 
 def train_epoch_base(model, train_loader, optimizer, device):
 	steps = 0
 	total_loss = torch.Tensor([0.0])
-	for batch in train_loader:
-		loss = do_train(model, batch, optimizer, device)
-		total_loss += loss
-		steps += 1
+	if hasattr(model, 'fit'):
+		for batch in train_loader:
+			loss = do_fit(model, batch, optimizer, device)
+			total_loss += loss
+			steps += 1
+	else:
+		for batch in train_loader:
+			loss = do_train(model, batch, optimizer, device)
+			total_loss += loss
+			steps += 1
 	
 	return total_loss.item() / steps
 
@@ -90,11 +149,18 @@ def train_epoch_progress(model, train_loader, optimizer, device, epoch, max_iter
 	steps = 0
 	total_loss = torch.Tensor([0.0])
 	loop = tqdm(train_loader, desc=f"[Epoch-{epoch}/{max_iter}]", total=len(train_loader), colour="green")
-	for batch in loop:
-		loss = do_train(model, batch, optimizer, device)
-		total_loss += loss
-		steps += 1
-		loop.set_postfix(loss=f"{total_loss.item() / steps:.4f}", lr=f'{optimizer.param_groups[0]["lr"]:.6f}')
+	if hasattr(model, 'fit'):
+		for batch in loop:
+			loss = do_fit(model, batch, optimizer, device)
+			total_loss += loss
+			steps += 1
+			loop.set_postfix(loss=f"{total_loss.item() / steps:.4f}", lr=f'{optimizer.param_groups[0]["lr"]:.6f}')
+	else:
+		for batch in loop:
+			loss = do_train(model, batch, optimizer, device)
+			total_loss += loss
+			steps += 1
+			loop.set_postfix(loss=f"{total_loss.item() / steps:.4f}", lr=f'{optimizer.param_groups[0]["lr"]:.6f}')
 	loop.close()
 	return total_loss.item() / steps
 
@@ -102,10 +168,16 @@ def train_epoch_progress(model, train_loader, optimizer, device, epoch, max_iter
 def train_epoch_scheduler(model, train_loader, optimizer, device, scheduler: LRScheduler):
 	steps = 0
 	total_loss = torch.Tensor([0.0])
-	for batch in train_loader:
-		loss = do_train_scheduler(model, batch, optimizer, device, scheduler)
-		total_loss += loss
-		steps += 1
+	if hasattr(model, 'fit'):
+		for batch in train_loader:
+			loss = do_fit_scheduler(model, batch, optimizer, device, scheduler)
+			total_loss += loss
+			steps += 1
+	else:
+		for batch in train_loader:
+			loss = do_train_scheduler(model, batch, optimizer, device, scheduler)
+			total_loss += loss
+			steps += 1
 	return total_loss.item() / steps
 
 
@@ -113,11 +185,18 @@ def train_epoch_scheduler_progress(model, train_loader, optimizer, device, sched
 	steps = 0
 	total_loss = torch.Tensor([0.0])
 	loop = tqdm(train_loader, desc=f"[Epoch-{epoch}/{max_iter}]", total=len(train_loader), colour="green")
-	for batch in loop:
-		loss = do_train_scheduler(model, batch, optimizer, device, scheduler)
-		total_loss += loss
-		steps += 1
-		loop.set_postfix(loss=f"{total_loss.item() / steps:.4f}", lr=f'{optimizer.param_groups[0]["lr"]:.6f}')
+	if hasattr(model, 'fit'):
+		for batch in loop:
+			loss = do_fit_scheduler(model, batch, optimizer, device, scheduler)
+			total_loss += loss
+			steps += 1
+			loop.set_postfix(loss=f"{total_loss.item() / steps:.4f}", lr=f'{optimizer.param_groups[0]["lr"]:.6f}')
+	else:
+		for batch in loop:
+			loss = do_train_scheduler(model, batch, optimizer, device, scheduler)
+			total_loss += loss
+			steps += 1
+			loop.set_postfix(loss=f"{total_loss.item() / steps:.4f}", lr=f'{optimizer.param_groups[0]["lr"]:.6f}')
 	loop.close()
 	return total_loss.item() / steps
 
@@ -136,12 +215,20 @@ def train_epoch(model, train_loader, optimizer, device, scheduler, epoch, max_it
 def train_epoch_base_acc(model, train_loader, optimizer, device):
 	total, steps = 0, 0
 	total_loss, total_correct = [torch.Tensor([0.0]) for _ in range(2)]
-	for batch in train_loader:
-		loss, count, correct = do_train_acc(model, batch, optimizer, device)
-		total_loss += loss
-		total += count
-		total_correct += correct
-		steps += 1
+	if hasattr(model, 'fit'):
+		for batch in train_loader:
+			loss, count, correct = do_fit_acc(model, batch, optimizer, device)
+			total_loss += loss
+			total += count
+			total_correct += correct
+			steps += 1
+	else:
+		for batch in train_loader:
+			loss, count, correct = do_train_acc(model, batch, optimizer, device)
+			total_loss += loss
+			total += count
+			total_correct += correct
+			steps += 1
 	return total_correct.item() / total, total_loss.item() / steps
 
 
@@ -149,14 +236,24 @@ def train_epoch_progress_acc(model, train_loader, optimizer, device, epoch, max_
 	total, steps = 0, 0
 	total_loss, total_correct = [torch.Tensor([0.0]) for _ in range(2)]
 	loop = tqdm(train_loader, desc=f"[Epoch-{epoch}/{max_iter}]", total=len(train_loader), colour="green")
-	for batch in loop:
-		loss, count, correct = do_train_acc(model, batch, optimizer, device)
-		total_loss += loss
-		total += count
-		total_correct += correct
-		steps += 1
-		loop.set_postfix(acc=f"{total_correct.item() / total:.4f}", loss=f"{total_loss.item() / steps:.4f}",
-		                 lr=f'{optimizer.param_groups[0]["lr"]:.6f}')
+	if hasattr(model, 'fit'):
+		for batch in loop:
+			loss, count, correct = do_fit_acc(model, batch, optimizer, device)
+			total_loss += loss
+			total += count
+			total_correct += correct
+			steps += 1
+			loop.set_postfix(acc=f"{total_correct.item() / total:.4f}", loss=f"{total_loss.item() / steps:.4f}",
+			                 lr=f'{optimizer.param_groups[0]["lr"]:.6f}')
+	else:
+		for batch in loop:
+			loss, count, correct = do_train_acc(model, batch, optimizer, device)
+			total_loss += loss
+			total += count
+			total_correct += correct
+			steps += 1
+			loop.set_postfix(acc=f"{total_correct.item() / total:.4f}", loss=f"{total_loss.item() / steps:.4f}",
+			                 lr=f'{optimizer.param_groups[0]["lr"]:.6f}')
 	loop.close()
 	
 	return total_correct.item() / total, total_loss.item() / steps
@@ -165,12 +262,20 @@ def train_epoch_progress_acc(model, train_loader, optimizer, device, epoch, max_
 def train_epoch_scheduler_acc(model, train_loader, optimizer, device, scheduler: LRScheduler):
 	total, steps = 0, 0
 	total_loss, total_correct = [torch.Tensor([0.0]) for _ in range(2)]
-	for batch in train_loader:
-		loss, count, correct = do_train_scheduler_acc(model, batch, optimizer, device, scheduler)
-		total_loss += loss
-		total += count
-		total_correct += correct
-		steps += 1
+	if hasattr(model, 'fit'):
+		for batch in train_loader:
+			loss, count, correct = do_fit_scheduler_acc(model, batch, optimizer, device, scheduler)
+			total_loss += loss
+			total += count
+			total_correct += correct
+			steps += 1
+	else:
+		for batch in train_loader:
+			loss, count, correct = do_train_scheduler_acc(model, batch, optimizer, device, scheduler)
+			total_loss += loss
+			total += count
+			total_correct += correct
+			steps += 1
 	return total_correct.item() / total, total_loss.item() / steps
 
 
@@ -178,14 +283,24 @@ def train_epoch_scheduler_progress_acc(model, train_loader, optimizer, device, s
 	total, steps = 0, 0
 	total_loss, total_correct = [torch.Tensor([0.0]) for _ in range(2)]
 	loop = tqdm(train_loader, desc=f"[Epoch-{epoch}/{max_iter}]", total=len(train_loader), colour="green")
-	for batch in loop:
-		loss, count, correct = do_train_scheduler_acc(model, batch, optimizer, device, scheduler)
-		total_loss += loss
-		total += count
-		total_correct += correct
-		steps += 1
-		loop.set_postfix(acc=f"{total_correct.item() / total:.4f}", loss=f"{total_loss.item() / steps:.4f}",
-		                 lr=f'{optimizer.param_groups[0]["lr"]:.6f}')
+	if hasattr(model, 'fit'):
+		for batch in loop:
+			loss, count, correct = do_fit_scheduler_acc(model, batch, optimizer, device, scheduler)
+			total_loss += loss
+			total += count
+			total_correct += correct
+			steps += 1
+			loop.set_postfix(acc=f"{total_correct.item() / total:.4f}", loss=f"{total_loss.item() / steps:.4f}",
+			                 lr=f'{optimizer.param_groups[0]["lr"]:.6f}')
+	else:
+		for batch in loop:
+			loss, count, correct = do_train_scheduler_acc(model, batch, optimizer, device, scheduler)
+			total_loss += loss
+			total += count
+			total_correct += correct
+			steps += 1
+			loop.set_postfix(acc=f"{total_correct.item() / total:.4f}", loss=f"{total_loss.item() / steps:.4f}",
+			                 lr=f'{optimizer.param_groups[0]["lr"]:.6f}')
 	loop.close()
 	
 	return total_correct.item() / total, total_loss.item() / steps
@@ -208,7 +323,7 @@ class Trainer:
 	             learning_rate=0.001, T_max: int = 0, batch_size=32, num_workers=0,
 	             pin_memory: bool = False, pin_memory_device: str = "", persistent_workers: bool = False,
 	             early_stopping_rounds: int = 2,  # 早停，等10轮决策，评价指标不在变化，停止
-	             print_per_rounds: int = 1, device=None
+	             print_per_rounds: int = 1, checkpoint_per_rounds: int = 0, checkpoint_name='model.pt', device=torch.device
 	             ):
 		self.max_iter = max_iter
 		self.optimizer = optimizer
@@ -222,9 +337,11 @@ class Trainer:
 		self.persistent_workers = persistent_workers
 		self.early_stopping_rounds = early_stopping_rounds
 		self.print_per_rounds = print_per_rounds
+		self.checkpoint_per_rounds = checkpoint_per_rounds
+		self.checkpoint_name = checkpoint_name
 		self.device = device or torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')
 	
-	def train(self, model, train_set: Dataset, collate_fn=None, show_progress=False):
+	def train(self, model, train_set: Dataset, collate_fn=None, show_progress=False, eps=1e-5):
 		train_loader = DataLoader(dataset=train_set, batch_size=self.batch_size, pin_memory=self.pin_memory,
 		                          pin_memory_device=self.pin_memory_device, persistent_workers=self.persistent_workers,
 		                          num_workers=self.num_workers, shuffle=True, collate_fn=collate_fn)
@@ -237,13 +354,11 @@ class Trainer:
 		for epoch in range(1, self.max_iter + 1):
 			avg_loss = train_epoch(model, train_loader, optimizer, self.device, scheduler, epoch, self.max_iter,
 			                       show_progress)
-			if self.print_per_rounds == 1:
-				self.print(epoch, optimizer.param_groups[0]["lr"], avg_loss)
-			elif self.print_per_rounds > 1:
-				if epoch % self.print_per_rounds == 0:
-					self.print(epoch, optimizer.param_groups[0]["lr"], avg_loss)
 			
-			if min_loss - avg_loss > E:
+			self.try_print(self.print, epoch, optimizer.param_groups[0]["lr"], avg_loss)
+			self.try_checkpoint(model, epoch)
+			
+			if min_loss - avg_loss > eps:
 				cnt = 0
 				best_model = deepcopy(model)
 				min_loss = avg_loss
@@ -275,6 +390,17 @@ class Trainer:
 		
 		return optimizer, scheduler
 	
+	def try_checkpoint(self, model, epoch):
+		if self.checkpoint_per_rounds <= 0:
+			return
+		
+		if self.checkpoint_per_rounds == 1 or epoch % self.checkpoint_per_rounds == 0:
+			torch.save(model, self.checkpoint_name)
+			
+	def try_print(self, do_print, epoch, lr, loss, **kwargs):
+		if self.print_per_rounds == 1 or epoch % self.print_per_rounds == 0:
+			do_print(epoch, lr, loss, **kwargs)
+			
 	def print(self, epoch, lr, loss):
 		log_utils.info(f'Epoch-{epoch}/{self.max_iter}  lr: {lr:.6f}, loss: {loss:.4f}')
 
@@ -286,17 +412,18 @@ class SimpleTrainer(Trainer):
 	             pin_memory: bool = False, pin_memory_device: str = "",
 	             persistent_workers: bool = False,
 	             early_stopping_rounds=10,  # 早停，等10轮决策，评价指标不在变化，停止
-	             print_per_rounds: int = 1, device=None):
+	             print_per_rounds: int = 1, checkpoint_per_rounds: int = 0, checkpoint_name='model.pt', device=torch.device):
 		super().__init__(max_iter, optimizer, scheduler, learning_rate, T_max, batch_size, num_workers, pin_memory,
-		                 pin_memory_device, persistent_workers, early_stopping_rounds, print_per_rounds, device)
+		                 pin_memory_device, persistent_workers, early_stopping_rounds, print_per_rounds, 
+		                 checkpoint_per_rounds, checkpoint_name, device)
 	
-	def train(self, model, X, y, collate_fn=None, show_progress=False):
+	def train(self, model, X, y, collate_fn=None, show_progress=False, eps=1e-5):
 		if isinstance(X, (List, np.ndarray)):
 			X = torch.tensor(X, dtype=torch.float)
 		if isinstance(y, (List, np.ndarray)):
 			y = torch.tensor(y, dtype=torch.long)
 		
-		return super().train(model, TensorDataset(X, y), collate_fn, show_progress)
+		return super().train(model, TensorDataset(X, y), collate_fn, show_progress, eps=eps)
 
 
 class ClassTrainer(Trainer):
@@ -306,11 +433,12 @@ class ClassTrainer(Trainer):
 	             pin_memory: bool = False, pin_memory_device: str = "",
 	             persistent_workers: bool = False,
 	             early_stopping_rounds=10,  # 早停，等10轮决策，评价指标不在变化，停止
-	             print_per_rounds: int = 1, device=None):
+	             print_per_rounds: int = 1, checkpoint_per_rounds: int = 0, checkpoint_name='model.pt', device=torch.device):
 		super().__init__(max_iter, optimizer, scheduler, learning_rate, T_max, batch_size, num_workers, pin_memory,
-		                 pin_memory_device, persistent_workers, early_stopping_rounds, print_per_rounds, device)
+		                 pin_memory_device, persistent_workers, early_stopping_rounds, print_per_rounds, 
+		                 checkpoint_per_rounds, checkpoint_name, device)
 	
-	def train(self, model, train_set: Dataset, collate_fn=None, show_progress=False):
+	def train(self, model, train_set: Dataset, collate_fn=None, show_progress=False, eps=1e-5):
 		train_loader = DataLoader(dataset=train_set, batch_size=self.batch_size, pin_memory=self.pin_memory,
 		                          pin_memory_device=self.pin_memory_device, persistent_workers=self.persistent_workers,
 		                          num_workers=self.num_workers, shuffle=True, collate_fn=collate_fn)
@@ -322,15 +450,12 @@ class ClassTrainer(Trainer):
 		model.train()
 		for epoch in range(1, self.max_iter + 1):
 			acc, avg_loss = train_epoch_acc(model, train_loader, optimizer, self.device, scheduler, epoch,
-			                                self.max_iter,
-			                                show_progress)
-			if self.print_per_rounds == 1:
-				self.print(epoch, optimizer.param_groups[0]["lr"], avg_loss, acc)
-			elif self.print_per_rounds > 1:
-				if epoch % self.print_per_rounds == 0:
-					self.print(epoch, optimizer.param_groups[0]["lr"], avg_loss, acc)
+			                                self.max_iter, show_progress)
 			
-			if acc > best_acc or (acc == best_acc and min_loss - avg_loss > E):
+			self.try_print(self.print, epoch, optimizer.param_groups[0]["lr"], avg_loss, acc=acc)
+			self.try_checkpoint(model, epoch)
+			
+			if acc > best_acc or (acc == best_acc and min_loss - avg_loss > eps):
 				cnt, cnt2 = 0, 0
 				best_acc, best_model = acc, deepcopy(model)
 				last_acc, min_loss = acc, min(min_loss, avg_loss)
@@ -341,7 +466,7 @@ class ClassTrainer(Trainer):
 				log_utils.info(f"Early stopping at Epoch-{epoch}/{self.max_iter}")
 				break
 			
-			if abs(last_acc - acc) < E:  # val_acc不在变化
+			if abs(last_acc - acc) < eps:  # val_acc不在变化
 				cnt2 += 1
 			else:
 				cnt2 = 0
@@ -364,17 +489,18 @@ class SimpleClassTrainer(ClassTrainer):
 	             pin_memory: bool = False, pin_memory_device: str = "",
 	             persistent_workers: bool = False,
 	             early_stopping_rounds=10,  # 早停，等10轮决策，评价指标不在变化，停止
-	             print_per_rounds: int = 1, device=None):
+	             print_per_rounds: int = 1, checkpoint_per_rounds: int = 0, checkpoint_name='model.pt', device=torch.device):
 		super().__init__(max_iter, optimizer, scheduler, learning_rate, T_max, batch_size, num_workers, pin_memory,
-		                 pin_memory_device, persistent_workers, early_stopping_rounds, print_per_rounds, device)
+		                 pin_memory_device, persistent_workers, early_stopping_rounds, print_per_rounds,
+		                 checkpoint_per_rounds, checkpoint_name, device)
 	
-	def train(self, model, X, y, collate_fn=None, show_progress=False):
+	def train(self, model, X, y, collate_fn=None, show_progress=False, eps=1e-5):
 		if isinstance(X, (List, np.ndarray)):
 			X = torch.tensor(X, dtype=torch.float)
 		if isinstance(y, (List, np.ndarray)):
 			y = torch.tensor(y, dtype=torch.long)
 		
-		return super().train(model, TensorDataset(X, y), collate_fn, show_progress)
+		return super().train(model, TensorDataset(X, y), collate_fn, show_progress, eps=eps)
 
 
 class EvalClassTrainer(Trainer):
@@ -383,13 +509,14 @@ class EvalClassTrainer(Trainer):
 	             learning_rate=0.001, T_max: int = 0, batch_size=32, eval_batch_size=64,
 	             num_workers=0, num_eval_workers=0, pin_memory: bool = False, pin_memory_device: str = "",
 	             persistent_workers: bool = False, early_stopping_rounds=10,  # 早停，等10轮决策，评价指标不在变化，停止
-	             print_per_rounds: int = 1, device=None):
+	             print_per_rounds: int = 1, checkpoint_per_rounds: int = 0, checkpoint_name='model.pt', device=torch.device):
 		super().__init__(max_iter, optimizer, scheduler, learning_rate, T_max, batch_size, num_workers, pin_memory,
-		                 pin_memory_device, persistent_workers, early_stopping_rounds, print_per_rounds, device)
+		                 pin_memory_device, persistent_workers, early_stopping_rounds, print_per_rounds,
+		                 checkpoint_per_rounds, checkpoint_name, device)
 		self.eval_batch_size = eval_batch_size
 		self.num_eval_workers = num_eval_workers
 	
-	def train(self, model, train_set: Dataset, val_set: Dataset, collate_fn=None, show_progress=False):
+	def train(self, model, train_set: Dataset, val_set: Dataset, collate_fn=None, show_progress=False, eps=1e-5):
 		train_loader = DataLoader(dataset=train_set, batch_size=self.batch_size, pin_memory=self.pin_memory,
 		                          pin_memory_device=self.pin_memory_device, persistent_workers=self.persistent_workers,
 		                          num_workers=self.num_workers, shuffle=True, collate_fn=collate_fn)
@@ -406,13 +533,12 @@ class EvalClassTrainer(Trainer):
 			                                self.max_iter, show_progress)
 			
 			val_loss, val_acc = evaluate(model, val_loader, self.device)
-			if self.print_per_rounds == 1:
-				self.print(epoch, optimizer.param_groups[0]["lr"], avg_loss, acc, val_loss, val_acc)
-			elif self.print_per_rounds > 1:
-				if epoch % self.print_per_rounds == 0:
-					self.print(epoch, optimizer.param_groups[0]["lr"], avg_loss, acc, val_loss, val_acc)
 			
-			if val_acc > best_acc or (val_acc == best_acc and min_loss - val_loss > E):
+			self.try_print(self.print, epoch, optimizer.param_groups[0]["lr"], avg_loss,
+			           acc=acc, val_loss=val_loss, val_acc=val_acc)
+			self.try_checkpoint(model, epoch)
+			
+			if val_acc > best_acc or (val_acc == best_acc and min_loss - val_loss > eps):
 				cnt, cnt2 = 0, 0
 				best_acc, best_model = val_acc, deepcopy(model)
 				last_acc, min_loss = val_acc, min(min_loss, val_loss)
@@ -423,7 +549,7 @@ class EvalClassTrainer(Trainer):
 				log_utils.info(f"Early stopping at Epoch-{epoch}/{self.max_iter}")
 				break
 			
-			if abs(last_acc - val_acc) < E:  # val_acc不在变化
+			if abs(last_acc - val_acc) < eps:  # val_acc不在变化
 				cnt2 += 1
 			else:
 				cnt2 = 0
@@ -449,12 +575,13 @@ class EvalSimpleClassTrainer(EvalClassTrainer):
 	             pin_memory: bool = False, pin_memory_device: str = "",
 	             persistent_workers: bool = False,
 	             early_stopping_rounds=10,  # 早停，等10轮决策，评价指标不在变化，停止
-	             print_per_rounds: int = 1, device=None):
+	             print_per_rounds: int = 1, checkpoint_per_rounds: int = 0, checkpoint_name='model.pt', device=torch.device):
+
 		super().__init__(max_iter, optimizer, scheduler, learning_rate, T_max, batch_size, eval_batch_size, num_workers,
 		                 num_eval_workers, pin_memory, pin_memory_device, persistent_workers, early_stopping_rounds,
-		                 print_per_rounds, device)
+		                 print_per_rounds, checkpoint_per_rounds, checkpoint_name, device)
 	
-	def train(self, model, X, y, val_data: Tuple, collate_fn=None, show_progress=False):
+	def train(self, model, X, y, val_data: Tuple, collate_fn=None, show_progress=False, eps=1e-5):
 		if isinstance(X, (List, np.ndarray)):
 			X = torch.tensor(X, dtype=torch.float)
 		if isinstance(y, (List, np.ndarray)):
@@ -465,7 +592,7 @@ class EvalSimpleClassTrainer(EvalClassTrainer):
 		if isinstance(y_val, (List, np.ndarray)):
 			y_val = torch.tensor(y_val, dtype=torch.long)
 		
-		return super().train(model, TensorDataset(X, y), TensorDataset(X_val, y_val), collate_fn, show_progress)
+		return super().train(model, TensorDataset(X, y), TensorDataset(X_val, y_val), collate_fn, show_progress, eps)
 
 
 class SplitClassTrainer(EvalSimpleClassTrainer):
@@ -476,11 +603,11 @@ class SplitClassTrainer(EvalSimpleClassTrainer):
 	             pin_memory: bool = False, pin_memory_device: str = "",
 	             persistent_workers: bool = False,
 	             early_stopping_rounds=10,  # 早停，等10轮决策，评价指标不在变化，停止
-	             print_per_rounds: int = 1, device=None):
+	             print_per_rounds: int = 1, checkpoint_per_rounds: int = 0, checkpoint_name='model.pt', device=torch.device):
 		super().__init__(max_iter, optimizer, scheduler, learning_rate, T_max, batch_size, eval_batch_size, num_workers,
 		                 num_eval_workers, pin_memory, pin_memory_device, persistent_workers, early_stopping_rounds,
-		                 print_per_rounds, device)
+		                 print_per_rounds, checkpoint_per_rounds, checkpoint_name, device)
 	
-	def train(self, model, X, y, val_size=0.2, random_state=None, collate_fn=None, show_progress=False):
+	def train(self, model, X, y, val_size=0.2, random_state=None, collate_fn=None, show_progress=False, eps=1e-5):
 		X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=val_size, random_state=random_state)
-		return super().train(model, X_train, y_train, (X_test, y_test), collate_fn, show_progress)
+		return super().train(model, X_train, y_train, (X_test, y_test), collate_fn, show_progress, eps)
